using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;
using System.Net;
using System.Net.Sockets;
using System.Globalization;
using System.Text;
using System.IO;
using System.Threading;
using System.Runtime.Serialization.Formatters.Soap;

namespace myseq {	

	// Settings singleton
	[Serializable]
	public sealed class Settings  {
		private volatile static Settings instance;
		private static object syncObj = new object();
	
		private string ip = "192.168.0.1";
		private int port = 5555;
		private bool beepOnAlert = true;
		private bool dontConPlayers = false;
		private bool prefixStars = true;
		private int rangeCircle = 150;
		private int levelOverride = -1;
		private Color backColor = Color.Black;
		private Size windowSize = new Size(896, 576);
		private Point windowLocation = new Point(0,0);
		private FormWindowState windowState = FormWindowState.Normal;
		private int splitterLoc = 256;
		private bool saveOnExit = true;
		private bool matchFullText = false;
		private int updateDelay = 250;
		private bool showPlayers = true;
		private bool showNPCs = true;
		private bool showCorpses = true;
		private bool drawDirectionLines = true;
		private bool depthFilter = false;
		public int c1w = 125;
		public int c2w = 40;
		public int c3w = 50;
		public int c4w = 50;
		public int c5w = 75;
		public int c6w = 50;
		public int c7w = 50;
		public int c8w = 75;
		public int c9w = 50;
		public int c10w = 100;
		public int c11w = 50;
		public int c12w = 50;
		public int c13w = 50;

		public bool DepthFilter  {
			get  {
				return depthFilter;
			}
			set  {
				depthFilter = value;
			}
		}

		public bool DrawDirectionLines  {
			get  {
				return drawDirectionLines;
			}
			set  {
				drawDirectionLines = value;
			}
		}

		public bool ShowCorpses  {
			get  {
				return showCorpses;
			}
			set  {
				showCorpses = value;
			}
		}

		public bool ShowNPCs  {
			get  {
				return showNPCs;
			}
			set  {
				showNPCs = value;
			}
		}

		public bool ShowPlayers  {
			get  {
				return showPlayers;
			}
			set  {
				showPlayers = value;
			}
		}

		public int UpdateDelay  {
			get  {
				return updateDelay;
			}
			set  {
				updateDelay = value;
			}
		}

		public bool MatchFullText  {
			get  {
				return matchFullText;
			}
			set  {
				matchFullText = value;
			}
		}

		public bool SaveOnExit  {
			get  {
				return saveOnExit;
			}
			set  {
				saveOnExit = value;
			}
		}

		public int SplitterLoc  {
			get  {
				return splitterLoc;
			}
			set  {
				splitterLoc = value;
			}
		}

		public FormWindowState WindowState  {
			get  {
				return windowState;
			}
			set  {
				windowState = value;
			}
		}

		public Size WindowSize  {
			get  {
				return windowSize;
			}
			set  {
				windowSize = value;
			}
		}

		public Point WindowLocation  {
			get  {
				return windowLocation;
			}
			set  {
				windowLocation = value;
			}
		}

		public Color BackColor  {
			get  {
				return backColor;
			}
			set  {
				backColor = value;
			}
		}

		public bool PrefixStars {
			get  {
				return prefixStars;
			}
			set  {
				prefixStars = value;
			}
		}

		public int LevelOverride {
			get  {
				return levelOverride;
			}
			set  {
				levelOverride = value;
			}
		}

		public int RangeCircle  {
			get  {
				return rangeCircle;
			}
			set  {
				rangeCircle = value;
			}
		}

		public bool BeepOnAlert  {
			get  {
				return beepOnAlert;
			}
			set  {
				beepOnAlert = value;
			}
		}

		public bool DontConPlayers  {
			get  {
				return dontConPlayers;
			}
			set  {
				dontConPlayers = value;
			}
		}

		public string IP  {
			get  {
				return ip;
			}
			set  {
				ip = value;
			}
		}
	
		public int Port  {
			get  {
				return port;
			}
			set  {
				port = value;
			}
		}

		private Settings()  {
		}

		public static Settings Instance {
			get {
				// only create a new instance if one doesn't already exist.
				if (instance == null) {
					// use this lock to ensure that only one thread is access
					// this block of code at once.
					lock(syncObj) {
						if (instance == null)
							instance = new Settings();
					}
				}
				// return instance where it was just created or already existed.
				return instance;
			}
			set {
				instance = value;
			}
		}

		public void Save(string filename)  {
			FileStream fs = new FileStream(filename, FileMode.Create);
			
			SoapFormatter sf1 = new SoapFormatter();

			
			sf1.Serialize(fs, Settings.Instance);

			fs.Close();
		}

		public void Load(string filename)  {
			FileStream fs = null;
			try  {
				fs = new FileStream(filename, FileMode.Open);
				
				SoapFormatter sf1 = new SoapFormatter();
				
				Settings.Instance = (Settings)sf1.Deserialize(fs);

			}
			catch  {
				// If there's an error the defaults will be used instead
			}
			if (fs != null)
				fs.Close();
		}
	}

	[StructLayout(LayoutKind.Sequential, Pack=1)]
	public class SPAWNINFO  {
		public SPAWNINFO()  {

		}


		public void frombytes(byte []b, int offset)  {
			Name = "";
			Name = Encoding.ASCII.GetString(b, 0+offset, 30);
			Y = BitConverter.ToSingle(b, 32+offset);
			X = BitConverter.ToSingle(b, 36+offset);
			Z = BitConverter.ToSingle(b, 40+offset);
			Heading = BitConverter.ToSingle(b, 44+offset);
			SpeedRun = BitConverter.ToSingle(b, 48+offset);
			SpawnID = BitConverter.ToUInt32(b, 52+offset);
			GuildID = BitConverter.ToInt16(b, 56+offset);
			Type = b[58+offset];
			Class = b[59+offset];
			Race = BitConverter.ToUInt32(b, 60+offset);
			Level = b[64+offset];
			Hide = b[65+offset];
			Light = b[66+offset];
			VisType = b[67+offset];
			Lastname = "";
			Lastname = Encoding.ASCII.GetString(b, 68+offset, 22);

			Deity = BitConverter.ToInt16(b, 90+offset);
			flags = BitConverter.ToInt32(b, 92+offset);
		}
		
		public string Name = "";
		public float Y = 0;
		public float X = 0;
		public float Z = 0;
		public float Heading = 0;
		public float SpeedRun = 0;
		public uint SpawnID = 0;
		public short GuildID = 0;
		public byte Type = 0;
		public byte Class = 0;
		public uint Race = 0;
		public byte Gender = 0;
		public byte Level = 0;
		public byte Hide; 		
		public byte Light = 0;
		public byte VisType = 0;
		public string Lastname = "";
		public short Deity = 0;
		public int gone = 0;
		public int flags = 0;
		public ListViewItem lvi = null;
		public bool isAlert = false;
	}

	public class GroundItem { 
		public float X = 0.0f;
		public float Y = 0.0f;
		public float Z = 0.0f;
		public uint ID = 0;
		public string Name = "";
		public int gone = 0;
	}

	[Serializable]
	public class ListBoxComparer : IComparer {
		//public ObjectComparer(params string[] fields) : this(fields, new bool[fields.Length]) {}

		public int Compare(object a, object b)  {
			ListViewItem sa = (ListViewItem)a;
			ListViewItem sb = (ListViewItem)b;
			int res = 0;

			// Name
			if (Column == 0)  {
				res = string.Compare(sa.Text,sb.Text);
			}
				// Level
			else if (Column == 1)  {
				int ia = int.Parse(sa.SubItems[1].Text);
				int ib = int.Parse(sb.SubItems[1].Text);
				
				if (ia < ib)
					res = -1;
				else if (ia > ib)
					res = 1;
				else
					res = 0;

			}	
				// Class
			else if (Column == 2)  {
				res = string.Compare(sa.SubItems[2].Text, sb.SubItems[2].Text);
			}
				// Race
			else if (Column == 3)  {
				res = string.Compare(sa.SubItems[3].Text, sb.SubItems[3].Text);
			}
				// Last Name
			else if (Column == 4)  {
				res = string.Compare(sa.SubItems[4].Text, sb.SubItems[4].Text);
			}
				// Type
			else if (Column == 5)  {
				res = string.Compare(sa.SubItems[5].Text, sb.SubItems[5].Text);
			}
				// Invis
			else if (Column == 5)  {
				res = string.Compare(sa.SubItems[6].Text, sb.SubItems[6].Text);
			}
				// Run Speed
			else if (Column == 6)  {
				float fa = float.Parse(sa.SubItems[7].Text);
				float fb = float.Parse(sb.SubItems[7].Text);
				
				if (fa < fb)
					res = -1;
				else if (fa > fb)
					res = 1;
				else
					res = 0;

			}	
				// SpawnID
			else if (Column == 7)  {
				int ia = int.Parse(sa.SubItems[8].Text);
				int ib = int.Parse(sb.SubItems[8].Text);
				
				if (ia < ib)
					res = -1;
				else if (ia > ib)
					res = 1;
				else
					res = 0;

			}	
			else if (Column == 8)  {
				DateTime dta = DateTime.Parse(sa.SubItems[9].Text);
				DateTime dtb = DateTime.Parse(sb.SubItems[9].Text);
	
				res = DateTime.Compare(dta, dtb);
			}
			

			if (Descending)
				res = -res;
			return res;
		}

		public ListBoxComparer(ListView.ListViewItemCollection spawns, bool descending, int column) {
			Spawns = spawns;
			Descending = descending;
			Column = column;
		}

		private ListView.ListViewItemCollection Spawns;
		private bool Descending;
		private int Column;
	}
		
		
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form  	{
		[DllImport("kernel32.dll", ExactSpelling=true)]
		internal static extern bool Beep(uint freq, uint dur);

		const int ditchGone = 2;
//		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.OpenFileDialog openFileDialog;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.MenuItem menuItem6;
		public System.Windows.Forms.ListView listBox1;
		private System.Windows.Forms.MenuItem menuItem8;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem menuItem10;
		private System.Windows.Forms.MenuItem menuItem11;
		private System.Windows.Forms.FontDialog fontDialog1;
		private System.Windows.Forms.MenuItem menuItem7;
		private System.Windows.Forms.Splitter splitter1;
		public myseq.MapPane mapPane1;


		public TcpClient tcpc = null;
		public NetworkStream s = null;
		public Thread myWorkerThread = null;
		public object mutex = new object();
		public ArrayList alerts = new ArrayList();
		public bool test = false;
		//public string addr = "127.0.0.1";
		public string curZone = "";
		public int curColumn = 0;
		public bool curDescend = false;
		public float rawScale = 1.0f;
		//public int port = 4734;

	//	public bool prefixStars = true;
		//public bool beepOnAlert = true;

		public string []Races = null;
		public string []Classes = null;
		private System.Windows.Forms.MenuItem menuItem12;
		private System.Windows.Forms.ColorDialog colorDialog1;
		private System.Windows.Forms.MenuItem menuItem13;
		private System.Windows.Forms.MenuItem menuItem14;
		private System.Windows.Forms.Timer timer1;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.MenuItem menuItem15;
		private System.Windows.Forms.ToolTip toolTip1;
		public string []Bodytypes = null;

		public void InitLookups()  {
			Classes = new string[33];
			Classes[0] = "Unknown";
			Classes[1] = "Warrior";
			Classes[2] = "Cleric";
			Classes[3] = "Paladin";
			Classes[4] = "Ranger";
			Classes[5] = "Shadow Knight";
			Classes[6] = "Druid";
			Classes[7] = "Monk";
			Classes[8] = "Bard";
			Classes[9] = "Rogue";
			Classes[10] = "Shaman";
			Classes[11] = "Necromancer";
			Classes[12] = "Wizard";
			Classes[13] = "Magician";
			Classes[14] = "Enchanter";
			Classes[15] = "Beastlord";
			Classes[16] = "Banker";
			Classes[17] = "Warrior GM";
			Classes[18] = "Cleric GM";
			Classes[19] = "Paladin GM";
			Classes[20] = "Ranger GM";
			Classes[21] = "Shandowknight GM";
			Classes[22] = "Druid GM";
			Classes[23] = "Monk GM";
			Classes[24] = "Bard GM";
			Classes[25] = "Rogue GM";
			Classes[26] = "Shaman GM";
			Classes[27] = "Necromancer GM";
			Classes[28] = "Wizard GM";
			Classes[29] = "Magician GM";
			Classes[30] = "Enchanter GM";
			Classes[31] = "Beastlord GM";
			Classes[32] = "Shopkeeper";

			Races = new string[400];
			Races[0] = "Unknown";
			Races[1] = "Human";
			Races[2] = "Barbarian";
			Races[3] = "Erudite";
			Races[4] = "Wood Elf";
			Races[5] = "High Elf";
			Races[6] = "Dark Elf";
			Races[7] = "Half Elf";
			Races[8] = "Dwarf";
			Races[9] = "Troll";
			Races[10] = "Ogre";
			Races[11] = "Halfling";
			Races[12] = "Gnome";
			Races[13] = "Aviak";
			Races[14] = "Werewolf";
			Races[15] = "Brownie";
			Races[16] = "Centaur";
			Races[17] = "Golem";
			Races[18] = "Giant/Cyclops";
			Races[19] = "Trakanon";
			Races[20] = "Venril Sathir";
			Races[21] = "Evil Eye";
			Races[22] = "Beetle";
			Races[23] = "Kerra";
			Races[24] = "Fish";
			Races[25] = "Fairy";
			Races[26] = "Froglok";
			Races[27] = "Froglok Ghoul";
			Races[28] = "Fungusman";
			Races[29] = "Gargoyle";
			Races[30] = "Gasbag";
			Races[31] = "Gel Cube";
			Races[32] = "Ghost";
			Races[33] = "Ghoul";
			Races[34] = "Giant Bat";
			Races[35] = "Giant Eel";
			Races[36] = "Giant Rat";
			Races[37] = "Giant Snake";
			Races[38] = "Giant Spider";
			Races[39] = "Gnoll";
			Races[40] = "Goblin";
			Races[41] = "Gorilla";
			Races[42] = "Wolf";
			Races[43] = "Bear";
			Races[44] = "Freeport Guard";
			Races[45] = "Demi Lich";
			Races[46] = "Imp";
			Races[47] = "Griffin";
			Races[48] = "Kobold";
			Races[49] = "Lava Dragon";
			Races[50] = "Lion";
			Races[51] = "Lizard Man";
			Races[52] = "Mimic";
			Races[53] = "Minotaur";
			Races[54] = "Orc";
			Races[55] = "Human Beggar";
			Races[56] = "Pixie";
			Races[57] = "Drachnid";
			Races[58] = "Solusek Ro";
			Races[59] = "Bloodgills";
			Races[60] = "Skeleton";
			Races[61] = "Shark";
			Races[62] = "Tunare";
			Races[63] = "Tiger";
			Races[64] = "Treant";
			Races[65] = "Vampire";
			Races[66] = "Rallos Zek";
			Races[67] = "Highpass Citizen";
			Races[68] = "Tentacle";
			Races[69] = "Will O' Wisp";
			Races[70] = "Zombie";
			Races[71] = "Qeynos Citizen";
			Races[72] = "Ship";
			Races[73] = "Launch";
			Races[74] = "Piranha";
			Races[75] = "Elemental";
			Races[76] = "Puma";
			Races[77] = "Neriak Citizen";
			Races[78] = "Erudin Citizen";
			Races[79] = "Bixie";
			Races[80] = "Reanimated Hand";
			Races[81] = "Rivervale Citizen";
			Races[82] = "Scarecrow";
			Races[83] = "Skunk";
			Races[84] = "Snake Elemental";
			Races[85] = "Spectre";
			Races[86] = "Sphinx";
			Races[87] = "Armadillo";
			Races[88] = "Clockwork Gnome";
			Races[89] = "Drake";
			Races[90] = "Halas Citizen";
			Races[91] = "Alligator";
			Races[92] = "Grobb Citizen";
			Races[93] = "Oggok Citizen";
			Races[94] = "Kaladim Citizen";
			Races[95] = "Cazic-Thule";
			Races[96] = "Cockatrice";
			Races[97] = "Daisy Man";
			Races[98] = "Elf Vampire";
			Races[99] = "Denizen";
			Races[100] = "Dervish";
			Races[101] = "Efreeti";
			Races[102] = "Froglok Tadpole";
			Races[103] = "Kedge";
			Races[104] = "Leech";
			Races[105] = "Swordfish";
			Races[106] = "Felwithe Guard";
			Races[107] = "Mammoth";
			Races[108] = "Eye of Zomm";
			Races[109] = "Wasp";
			Races[110] = "Mermaid";
			Races[111] = "Harpie";
			Races[112] = "Faydark Guard";
			Races[113] = "Drixie";
			Races[114] = "Ghost Ship";
			Races[115] = "Clam";
			Races[116] = "Sea Horse";
			Races[117] = "Dwarf Ghost";
			Races[118] = "Erudite Ghost";
			Races[119] = "Sabertooth Cat";
			Races[120] = "Wolf Elemental";
			Races[121] = "Gorgon";
			Races[122] = "Dragon Skeleton";
			Races[123] = "Innoruuk";
			Races[124] = "Unicorn";
			Races[125] = "Pegasus";
			Races[126] = "Djinn";
			Races[127] = "Invisible Man";
			Races[128] = "Iksar";
			Races[129] = "Scorpion";
			Races[130] = "Vah Shir";
			Races[131] = "Sarnak";
			Races[132] = "Draglock";
			Races[133] = "Drolvarg";
			Races[134] = "Mosquito";
			Races[135] = "Rhino";
			Races[136] = "Xalgoz";
			Races[137] = "Kunark Goblin";
			Races[138] = "Yeti";
			Races[139] = "Iksar Citizen";
			Races[140] = "Forest Giant";
			Races[141] = "Boat";
			Races[142] = "Burynai";
			Races[143] = "Unknown";
			Races[144] = "Unknown";
			Races[145] = "Goo";
			Races[146] = "Spectral Sarnak";
			Races[147] = "Spectral Iksar";
			Races[148] = "Kunark Fish";
			Races[149] = "Iksar Scorpion";
			Races[150] = "Erollisi";
			Races[151] = "Tribunal";
			Races[152] = "Bertoxxulous";
			Races[153] = "Bristlebane";
			Races[154] = "Fay Drake";
			Races[155] = "Sarnak Skeleton";
			Races[156] = "Ratman";
			Races[157] = "Wyvern";
			Races[158] = "Wurm";
			Races[159] = "Devourer";
			Races[160] = "Iksar Golem";
			Races[161] = "Iksar Skeleton";
			Races[162] = "Man Eating Plant";
			Races[163] = "Raptor";
			Races[164] = "Sarnak Golem";
			Races[165] = "Water Dragon";
			Races[166] = "Iksar Hand";
			Races[167] = "Succulent";
			Races[168] = "Flying Monkey";
			Races[169] = "Brontotherium";
			Races[170] = "Snow Dervish";
			Races[171] = "Dire Wolf";
			Races[172] = "Manticore";
			Races[173] = "Totem";
			Races[174] = "Cold Spectre";
			Races[175] = "Enchanted Armor";
			Races[176] = "Snow Bunny";
			Races[177] = "Walrus";
			Races[178] = "Rock-Gem Men";
			Races[179] = "Unknown";
			Races[180] = "Unknown";
			Races[181] = "Yak Man";
			Races[182] = "Faun";
			Races[183] = "Coldain";
			Races[184] = "Velious Dragon";
			Races[185] = "Hag";
			Races[186] = "Hippogriff";
			Races[187] = "Siren";
			Races[188] = "Frost Giant";
			Races[189] = "Storm Giant";
			Races[190] = "Ottermen";
			Races[191] = "Walrus Man";
			Races[192] = "Clockwork Dragon";
			Races[193] = "Abhorent";
			Races[194] = "Sea Turtle";
			Races[195] = "B&W Dragon";
			Races[196] = "Ghost Dragon";
			Races[197] = "Ronnie Test";
			Races[198] = "Prismatic Dragon";
			Races[199] = "ShikNar";
			Races[200] = "Rockhopper";
			Races[201] = "Underbulk";
			Races[202] = "Grimling";
			Races[203] = "Vacuum Worm";
			Races[204] = "Evan Test";
			Races[205] = "Kahli Shah";
			Races[206] = "Owlbear";
			Races[207] = "Rhino Beetle";
			Races[208] = "Vampyre";
			Races[209] = "Earth Elemental";
			Races[210] = "Air Elemental";
			Races[211] = "Water Elemental";
			Races[212] = "Fire Elemental";
			Races[213] = "Wetfang Minnow";
			Races[214] = "Thought Horror";
			Races[215] = "Tegi";
			Races[216] = "Horse";
			Races[217] = "Shissar";
			Races[218] = "Fungal Fiend";
			Races[219] = "Vampire Volatalis";
			Races[220] = "StoneGrabber";
			Races[221] = "Scarlet Cheetah";
			Races[222] = "Zelniak";
			Races[223] = "Lightcrawler";
			Races[224] = "Shade";
			Races[225] = "Sunflower";
			Races[226] = "Sun Revenant";
			Races[227] = "Shrieker";
			Races[228] = "Galorian";
			Races[229] = "Netherbian";
			Races[230] = "Akheva";
			Races[231] = "Spire Spirit";
			Races[232] = "Sonic Wolf";
			Races[233] = "Teleport Man";
			Races[234] = "Vah Shir Skeleton";
			Races[235] = "Mutant Humanoid";
			Races[236] = "Seru";
			Races[237] = "Recuso";
			Races[238] = "Vah Shir King";
			Races[239] = "Vah Shir Guard";
			Races[240] = "Teleport Man";
			Races[241] = "Lujein";
			Races[242] = "Naiad";
			Races[243] = "Nymph";
			Races[244] = "Ent";
			Races[245] = "Wrinnfly";
			Races[246] = "Tarew Marr";
			Races[247] = "Solusek Ro";
			Races[248] = "Clockwork Golem";
			Races[249] = "Clockwork Brain";
			Races[250] = "Spectral Banshee";
			Races[251] = "Guard of Justice";
			Races[252] = "PoM Castle";
			Races[253] = "Disease Boss";
			Races[254] = "Solusek Ro Guard";
			Races[255] = "Bertoxxulous";
			Races[256] = "New Tribunal";
			Races[257] = "Terris Thule";
			Races[258] = "Vegerog";
			Races[259] = "Crocodile";
			Races[260] = "Bat";
			Races[261] = "Slarghilug";
			Races[262] = "Tranquilion";
			Races[263] = "Tin Soldier";
			Races[264] = "Nightmare Wraith";
			Races[265] = "Malarian";
			Races[266] = "Knight of Pestilence";
			Races[267] = "Lepertoloth";
			Races[268] = "Bubonian Boss";
			Races[269] = "Bubonian Underling";
			Races[270] = "Pusling";
			Races[271] = "Water Mephit";
			Races[272] = "Stormrider";
			Races[273] = "Junk Beast";
			Races[274] = "Broken Clockwork";
			Races[275] = "Giant Clockwork";
			Races[276] = "Clockwork Beetle";
			Races[277] = "Nightmare Goblin";
			Races[278] = "Karana";
			Races[279] = "Blood Raven";
			Races[280] = "Nightmare Gargoyle";
			Races[281] = "Mouths of Insanity";
			Races[282] = "Skeletal Horse";
			Races[283] = "Saryn";
			Races[284] = "Fennin Ro";
			Races[285] = "Tormentor";
			Races[286] = "Necromancer Priest";
			Races[287] = "Nightmare";
			Races[288] = "New Rallos Zek";
			Races[289] = "Vallon Zek";
			Races[290] = "Tallon Zek";
			Races[291] = "Air Mephit";
			Races[292] = "Earth Mephit";
			Races[293] = "Fire Mephit";
			Races[294] = "Nightmare Mephit";
			Races[295] = "Zebukoruk";
			Races[296] = "Mithaniel Marr";
			Races[297] = "Knightmare";
			Races[298] = "The Rathe";
			Races[299] = "Xegony";
			Races[300] = "Balrog";
			Races[301] = "Unknown";
			Races[302] = "Lobster Monster";
			Races[303] = "Pheonix";
			Races[304] = "Tiamat";
			Races[305] = "Bear PoP";
			Races[306] = "Earth Golem";
			Races[307] = "Iron Golem";
			Races[308] = "Storm Golem";
			Races[309] = "Air Golem";
			Races[310] = "Wood Golem";
			Races[311] = "Fire Golem";
			Races[312] = "Water Golem";
			Races[313] = "Veiled Gargoyle";
			Races[314] = "Lynx";
			Races[315] = "Squid";
			Races[316] = "Frog";
			Races[317] = "Flying Serpent";
			Races[318] = "War Soldier";
			Races[319] = "Armored Boar";
			Races[320] = "Djinni";
			Races[321] = "Boar";
			Races[322] = "Armor Knight";
			Races[323] = "Ghost Armor";
			Races[324] = "Death Knight";
			Races[325] = "Zek Ogre";
			Races[326] = "Nightmare Spider";
			Races[327] = "Crystal Spider";
			Races[328] = "A Tower";
			Races[329] = "Unknown";
			Races[330] = "Froglok";
			Races[331] = "Unknown";
			Races[332] = "Unknown";
			Races[333] = "Unknown";
			Races[334] = "Unknown";
			Races[335] = "Unknown";
			Races[336] = "Unknown";
			Races[337] = "Unknown";
			Races[338] = "Unknown";
			Races[339] = "Unknown";
			Races[340] = "Unknown";
			Races[341] = "Unknown";
			Races[342] = "Unknown";
			Races[343] = "Unknown";
			Races[344] = "Unknown";
			Races[345] = "Unknown";
			Races[346] = "Unknown";
			Races[347] = "Unknown";
			Races[348] = "Unknown";
			Races[349] = "Unknown";
			Races[350] = "Unknown";
			Races[351] = "Unknown";
			Races[352] = "Unknown";
			Races[353] = "Unknown";
			Races[354] = "Unknown";
			Races[355] = "Unknown";
			Races[356] = "Unknown";
			Races[357] = "Unknown";
			Races[358] = "Unknown";
			Races[359] = "Unknown";
			Races[360] = "Unknown";
			Races[361] = "Unknown";
			Races[362] = "Unknown";
			Races[363] = "Unknown";
			Races[364] = "Unknown";
			Races[365] = "Unknown";
			Races[366] = "Unknown";
			Races[367] = "Unknown";
			Races[368] = "Unknown";
			Races[369] = "Unknown";
			Races[370] = "Unknown";
			Races[371] = "Unknown";
			Races[372] = "Unknown";
			Races[373] = "Unknown";
			Races[374] = "Unknown";
			Races[375] = "Unknown";
			Races[376] = "Unknown";
			Races[377] = "Unknown";
			Races[378] = "Unknown";
			Races[379] = "Unknown";
			Races[380] = "Unknown";
			Races[381] = "Unknown";
			Races[382] = "Unknown";
			Races[383] = "Unknown";
			Races[384] = "Unknown";
			Races[385] = "Unknown";
			Races[386] = "Unknown";
			Races[387] = "Unknown";
			Races[388] = "Unknown";
			Races[389] = "Unknown";
			Races[390] = "Unknown";
			Races[391] = "Unknown";
			Races[392] = "Unknown";
			Races[393] = "Unknown";
			Races[394] = "Unknown";
			Races[395] = "Unknown";
			Races[396] = "Unknown";
			Races[397] = "Unknown";
			Races[398] = "Unknown";
			Races[399] = "Unknown";
			
			Bodytypes = new string[69];
			Bodytypes[0] = "Player";
			Bodytypes[1] = "Humanoid";
			Bodytypes[2] = "Corpse";
			Bodytypes[3] = "Undead";
			Bodytypes[4] = "Giant";
			Bodytypes[5] = "Golem";
			Bodytypes[6] = "UndeadPet";
			Bodytypes[7] = "BaneGiant";
			Bodytypes[8] = "Unknown Type #8";
			Bodytypes[9] = "Unknown Type #9";
			Bodytypes[10] = "Unknown Type #10";
			Bodytypes[11] = "Untargetable";
			Bodytypes[12] = "Unknown Type #12";
			Bodytypes[13] = "Unknown Type #13";
			Bodytypes[14] = "Unknown Type #14";
			Bodytypes[15] = "Unknown Type #15";
			Bodytypes[16] = "Seru";
			Bodytypes[17] = "Unknown Type #17";
			Bodytypes[18] = "Unknown Type #18";
			Bodytypes[19] = "Zek";
			Bodytypes[20] = "Unknown Type #20";
			Bodytypes[21] = "Animal";
			Bodytypes[22] = "Insect";
			Bodytypes[23] = "Unknown Type #23";
			Bodytypes[24] = "Summoned";
			Bodytypes[25] = "Plant";
			Bodytypes[26] = "Dragonkin";
			Bodytypes[27] = "Elemental";
			Bodytypes[28] = "Doppelganger";
			Bodytypes[29] = "Dragon";
			Bodytypes[30] = "BaneDragon";
			Bodytypes[31] = "Familiar";
			Bodytypes[32] = "Unknown Type #32";
			Bodytypes[33] = "Unknown Type #33";
			Bodytypes[34] = "Unknown Type #34";
			Bodytypes[35] = "Unknown Type #35";
			Bodytypes[36] = "Unknown Type #36";
			Bodytypes[37] = "Unknown Type #37";
			Bodytypes[38] = "Unknown Type #38";
			Bodytypes[39] = "Unknown Type #39";
			Bodytypes[40] = "Unknown Type #40";
			Bodytypes[41] = "Unknown Type #41";
			Bodytypes[42] = "Unknown Type #42";
			Bodytypes[43] = "Unknown Type #43";
			Bodytypes[44] = "Unknown Type #44";
			Bodytypes[45] = "Unknown Type #45";
			Bodytypes[46] = "Unknown Type #46";
			Bodytypes[47] = "Unknown Type #47";
			Bodytypes[48] = "Unknown Type #48";
			Bodytypes[49] = "Unknown Type #49";
			Bodytypes[50] = "Unknown Type #50";
			Bodytypes[51] = "Unknown Type #51";
			Bodytypes[52] = "Unknown Type #52";
			Bodytypes[53] = "Unknown Type #53";
			Bodytypes[54] = "Unknown Type #54";
			Bodytypes[55] = "Unknown Type #55";
			Bodytypes[56] = "Unknown Type #56";
			Bodytypes[57] = "Unknown Type #57";
			Bodytypes[58] = "Unknown Type #58";
			Bodytypes[59] = "Unknown Type #59";
			Bodytypes[60] = "Unknown Type #60";
			Bodytypes[61] = "Unknown Type #61";
			Bodytypes[62] = "Unknown Type #62";
			Bodytypes[63] = "Unknown Type #63";
			Bodytypes[64] = "Unknown Type #64";
			Bodytypes[65] = "Trap";
			Bodytypes[66] = "Timer";
			Bodytypes[67] = "EventTrigger";
			Bodytypes[68] = "Unknown Type #68";
		}

		public Form1()  {
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			
			loadPrefs("prefs.xml");
			

			listBox1.View = View.Details;

			listBox1.AllowColumnReorder = true;
			listBox1.FullRowSelect = true;
			listBox1.GridLines = true;
			listBox1.MultiSelect = false;
			

			listBox1.Columns.Add("Name", Settings.Instance.c1w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Level", Settings.Instance.c2w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Class", Settings.Instance.c3w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Race", Settings.Instance.c4w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Last Name", Settings.Instance.c5w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Type", Settings.Instance.c6w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Invis", Settings.Instance.c7w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Run Speed", Settings.Instance.c8w, HorizontalAlignment.Left);
			listBox1.Columns.Add("SpawnID", Settings.Instance.c9w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Spawn Time", Settings.Instance.c10w, HorizontalAlignment.Left);
			listBox1.Columns.Add("X", Settings.Instance.c11w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Y", Settings.Instance.c12w, HorizontalAlignment.Left);
			listBox1.Columns.Add("Z", Settings.Instance.c13w, HorizontalAlignment.Left);

			lock(mapPane1.map.mobs)  
				mapPane1.map.mobs.Clear();
			lock(mapPane1.map.items)
				mapPane1.map.items.Clear();
			lock(listBox1)  
				listBox1.Items.Clear();

			listBox1.ListViewItemSorter = new ListBoxComparer(listBox1.Items, false, 0);

			loadAlerts("filters.conf");
			InitLookups();

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )  {

			if( disposing )  {
				//if (components != null)  {
				//	components.Dispose();
				//}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.menuItem14 = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.menuItem5 = new System.Windows.Forms.MenuItem();
			this.menuItem8 = new System.Windows.Forms.MenuItem();
			this.menuItem7 = new System.Windows.Forms.MenuItem();
			this.menuItem12 = new System.Windows.Forms.MenuItem();
			this.menuItem13 = new System.Windows.Forms.MenuItem();
			this.menuItem15 = new System.Windows.Forms.MenuItem();
			this.menuItem9 = new System.Windows.Forms.MenuItem();
			this.menuItem6 = new System.Windows.Forms.MenuItem();
			this.menuItem10 = new System.Windows.Forms.MenuItem();
			this.menuItem11 = new System.Windows.Forms.MenuItem();
			this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.listBox1 = new System.Windows.Forms.ListView();
			this.fontDialog1 = new System.Windows.Forms.FontDialog();
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.mapPane1 = new myseq.MapPane();
			this.colorDialog1 = new System.Windows.Forms.ColorDialog();
			this.timer1 = new System.Windows.Forms.Timer(this.components);
			this.toolTip1 = new System.Windows.Forms.ToolTip(this.components);
			this.SuspendLayout();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem1,
																					  this.menuItem4,
																					  this.menuItem9});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem2,
																					  this.menuItem14,
																					  this.menuItem3});
			this.menuItem1.Text = "&File";
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 0;
			this.menuItem2.Text = "Open &Map";
			this.menuItem2.Click += new System.EventHandler(this.menuItem2_Click);
			// 
			// menuItem14
			// 
			this.menuItem14.Index = 1;
			this.menuItem14.Text = "&Save Mobs";
			this.menuItem14.Click += new System.EventHandler(this.menuItem14_Click);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 2;
			this.menuItem3.Text = "E&xit";
			this.menuItem3.Click += new System.EventHandler(this.menuItem3_Click);
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 1;
			this.menuItem4.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem5,
																					  this.menuItem8,
																					  this.menuItem7,
																					  this.menuItem12,
																					  this.menuItem13,
																					  this.menuItem15});
			this.menuItem4.Text = "&Options";
			// 
			// menuItem5
			// 
			this.menuItem5.Index = 0;
			this.menuItem5.Text = "&Options";
			this.menuItem5.Click += new System.EventHandler(this.menuItem5_Click);
			// 
			// menuItem8
			// 
			this.menuItem8.Index = 1;
			this.menuItem8.Shortcut = System.Windows.Forms.Shortcut.F2;
			this.menuItem8.Text = "Reset Mobs";
			this.menuItem8.Click += new System.EventHandler(this.menuItem8_Click);
			// 
			// menuItem7
			// 
			this.menuItem7.Index = 2;
			this.menuItem7.Text = "List &Font";
			this.menuItem7.Click += new System.EventHandler(this.menuItem7_Click);
			// 
			// menuItem12
			// 
			this.menuItem12.Index = 3;
			this.menuItem12.Text = "&Background Color";
			this.menuItem12.Click += new System.EventHandler(this.menuItem12_Click);
			// 
			// menuItem13
			// 
			this.menuItem13.Index = 4;
			this.menuItem13.Text = "&Save Preferences";
			this.menuItem13.Click += new System.EventHandler(this.menuItem13_Click);
			// 
			// menuItem15
			// 
			this.menuItem15.Index = 5;
			this.menuItem15.Shortcut = System.Windows.Forms.Shortcut.F3;
			this.menuItem15.Text = "&Depth Filter";
			this.menuItem15.Click += new System.EventHandler(this.menuItem15_Click);
			// 
			// menuItem9
			// 
			this.menuItem9.Index = 2;
			this.menuItem9.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem6,
																					  this.menuItem10,
																					  this.menuItem11});
			this.menuItem9.Text = "&Alerts";
			// 
			// menuItem6
			// 
			this.menuItem6.Index = 0;
			this.menuItem6.Text = "Reload Alerts";
			this.menuItem6.Click += new System.EventHandler(this.menuItem6_Click);
			// 
			// menuItem10
			// 
			this.menuItem10.Index = 1;
			this.menuItem10.Text = "&Add Alert";
			this.menuItem10.Click += new System.EventHandler(this.menuItem10_Click);
			// 
			// menuItem11
			// 
			this.menuItem11.Index = 2;
			this.menuItem11.Text = "&Save Alerts";
			this.menuItem11.Click += new System.EventHandler(this.menuItem11_Click);
			// 
			// listBox1
			// 
			this.listBox1.Dock = System.Windows.Forms.DockStyle.Left;
			this.listBox1.Font = new System.Drawing.Font("Microsoft Sans Serif", 7F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.listBox1.FullRowSelect = true;
			this.listBox1.GridLines = true;
			this.listBox1.HideSelection = false;
			this.listBox1.Name = "listBox1";
			this.listBox1.Size = new System.Drawing.Size(352, 529);
			this.listBox1.TabIndex = 15;
			this.listBox1.View = System.Windows.Forms.View.Details;
			this.listBox1.ColumnClick += new System.Windows.Forms.ColumnClickEventHandler(this.listBox1_ColumnClick);
			this.listBox1.SelectedIndexChanged += new System.EventHandler(this.listBox1_SelectedIndexChanged);
			// 
			// fontDialog1
			// 
			this.fontDialog1.Apply += new System.EventHandler(this.fontDialog1_Apply_1);
			// 
			// splitter1
			// 
			this.splitter1.Location = new System.Drawing.Point(352, 0);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(3, 529);
			this.splitter1.TabIndex = 16;
			this.splitter1.TabStop = false;
			this.splitter1.SplitterMoved += new System.Windows.Forms.SplitterEventHandler(this.Splitter1_SplitterMoved);
			// 
			// mapPane1
			// 
			this.mapPane1.BackColor = System.Drawing.SystemColors.Control;
			this.mapPane1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.mapPane1.Location = new System.Drawing.Point(355, 0);
			this.mapPane1.Name = "mapPane1";
			this.mapPane1.Size = new System.Drawing.Size(533, 529);
			this.mapPane1.TabIndex = 17;
			// 
			// timer1
			// 
			this.timer1.Tick += new System.EventHandler(this.timer1_Tick);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(888, 529);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.mapPane1,
																		  this.splitter1,
																		  this.listBox1});
			this.Menu = this.mainMenu1;
			this.Name = "Form1";
			this.StartPosition = System.Windows.Forms.FormStartPosition.WindowsDefaultBounds;
			this.Text = "MyShowEQ version 1.10 - ";
			this.Resize += new System.EventHandler(this.Form1_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.Form1_Closing);
			this.Load += new System.EventHandler(this.Form1_Load);
			this.Move += new System.EventHandler(this.Form1_Move);
			this.Closed += new System.EventHandler(this.form_Closing);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {
			//try  {
				Application.Run(new Form1());
			/*}
			catch  {
				MessageBox.Show("Uncaught exception...\n");
				Application.Exit();
			}*/
		}

		private void form_Closing(object sender, System.EventArgs e)  {
			if (myWorkerThread != null)
				myWorkerThread.Abort();
		}


		public void button1_Click(object sender, System.EventArgs e)  {
			//Byte[] read = new Byte[366];
			IPAddress ip = null;

			if (mapPane1.button1.Text == "GO")  {
				// Try to connect to the server				
				try  {
					ip = IPAddress.Parse(Settings.Instance.IP);
				}
				catch  {
					MessageBox.Show("Invalid IP address", "Error");
					return;
				}

				tcpc = new TcpClient();

				try  {
					tcpc.Connect(ip, Settings.Instance.Port);

					// Get the stream
					s = tcpc.GetStream();
				} catch {
					MessageBox.Show("Cannot connect to server", "Error");					
					return;
				}

				//button1.Enabled = false;

				ThreadStart myThreadStart = new ThreadStart(this.NetThread);
				myWorkerThread = new Thread(myThreadStart);

				myWorkerThread.Start();
				mapPane1.button1.Text = "Stop";
			}
			else  {
				myWorkerThread.Abort();

				s.Close();
				tcpc.Close();

				//mapPane1.map.mobs.Clear();
				//listBox1.Items.Clear();
			
				mapPane1.button1.Text = "GO";
			}

		}

		public void loadPrefs(string filename)  {
			Settings.Instance.Load(filename);
			
//			this.menuItem16.Checked = Settings.Instance.ShowNPCs;
//			this.menuItem17.Checked = Settings.Instance.ShowPlayers;
//			this.menuItem18.Checked = Settings.Instance.ShowCorpses;

			this.WindowState = Settings.Instance.WindowState;
			this.Size = Settings.Instance.WindowSize;
			this.Location = Settings.Instance.WindowLocation;

			this.splitter1.SplitPosition = Settings.Instance.SplitterLoc;
		}

		public void savePrefs(string filename)  {
			Settings.Instance.c1w = listBox1.Columns[0].Width;
			Settings.Instance.c2w = listBox1.Columns[1].Width;
			Settings.Instance.c3w = listBox1.Columns[2].Width;
			Settings.Instance.c4w = listBox1.Columns[3].Width;
			Settings.Instance.c5w = listBox1.Columns[4].Width;
			Settings.Instance.c6w = listBox1.Columns[5].Width;
			Settings.Instance.c7w = listBox1.Columns[6].Width;
			Settings.Instance.c8w = listBox1.Columns[7].Width;
			Settings.Instance.c9w = listBox1.Columns[8].Width;
			Settings.Instance.c10w = listBox1.Columns[9].Width;
			Settings.Instance.c11w = listBox1.Columns[10].Width;
			Settings.Instance.c12w = listBox1.Columns[11].Width;
			Settings.Instance.c13w = listBox1.Columns[12].Width;
			Settings.Instance.Save(filename);			
		}

		public void saveAlerts(string filename)  {			
			// Load the alerts
			StreamWriter sw;
			try  {
				sw = new StreamWriter(File.Open(filename, FileMode.Create));
			}
			catch (System.IO.FileNotFoundException)  {
				Console.WriteLine("File not found");
				return ;
			}
			catch (System.Exception)  {
				Console.WriteLine("Unknown exception caught while opening source file");
				return ;
			}  

			foreach (string s in alerts)  {
				sw.WriteLine(s);
			}

			sw.Close();
		}

		public void loadAlerts(string filename)  {
			alerts.Clear();

			// Load the alerts
			StreamReader sw;
			try  {
				sw = new StreamReader(File.Open(filename, FileMode.Open));
			}
			catch (System.IO.FileNotFoundException)  {
				Console.WriteLine("File not found");
				return ;
			}
			catch (System.Exception)  {
				Console.WriteLine("Unknown exception caught while opening source file");
				return ;
			}  

			string inp = "";
			while ((inp = sw.ReadLine()) != null)   {
				alerts.Add(inp);
			}			
			sw.Close();
			
		}		

		public bool isSubstring(string a, string b)  {			
			int c;

			for (c = 0; c < a.Length-b.Length; c++)  {
				if (string.Compare(a, c, b, 0, b.Length, true) == 0)
					return true;
			}

			return false;
		}

		public bool loadmap(string filename)  {

			// Decide if the LoY or SEQ map loader should be used
			//spook 2003-04-26: changed app title header to indicate when a map was not found
			if (filename.EndsWith(".map"))  
			{
				if (!mapPane1.map.loadMap(filename))  {
					this.Text = "MyShowEQ  version 1.10 - MAP NOT FOUND";
					return false;
				}
			}
			else if (filename.EndsWith(".txt"))  {
				if (!mapPane1.map.loadLoYMap(filename))  {
					this.Text = "MyShowEQ  version 1.10 - MAP NOT FOUND";
					return false;
				}
			}
			
			this.Text = "MyShowEQ  version 1.10 - " + mapPane1.map.longname;

			lock(mapPane1.map.mobs)  
				mapPane1.map.mobs.Clear();
			lock(mapPane1.map.items)
				mapPane1.map.items.Clear();
			lock(listBox1)  
				listBox1.Items.Clear();

			mapPane1.scale.Value = 100M;
			return true;
		}

		private void menuItem3_Click(object sender, System.EventArgs e) {
			this.Close();
			Application.Exit();
		}

		private void menuItem2_Click(object sender, System.EventArgs e) {
			openFileDialog.Filter = "Map Files (*.map)|*.map|LoY Map Files (*.txt)|*.txt|All Files (*.*)|(*.*)";

			if (openFileDialog.ShowDialog() == DialogResult.OK)  {	// Show the standard open file dialog
				string filename = openFileDialog.FileName;			
				loadmap(filename);
			}
		}

		private void Splitter1_SplitterMoved(object sender, SplitterEventArgs e)  {
			Settings.Instance.SplitterLoc = splitter1.SplitPosition;
		}

		private void Form1_Move(object sender, EventArgs e)  {
			if (WindowState == FormWindowState.Normal)
				Settings.Instance.WindowLocation = this.Location;
		}

		private void Form1_Resize(object sender, EventArgs e) {
			if (WindowState == FormWindowState.Normal)
				Settings.Instance.WindowSize = this.Size;
			Settings.Instance.WindowState = this.WindowState;
		}

		public int strlen(string str) { 
			int c = 0;

			while (c < str.Length && str[c] != 0 )
				c++;

			return c;
		}

		private void mapCon1_Load(object sender, System.EventArgs e) {
		
		}

		private void menuItem5_Click(object sender, System.EventArgs e) {
			Form3 f3 = new Form3();
			MapCon mc = mapPane1.map;

			f3.textBox1.Text = Settings.Instance.IP;
			f3.textBox2.Text = Settings.Instance.Port.ToString();
			f3.checkBox1.Checked = Settings.Instance.BeepOnAlert;
			f3.checkBox2.Checked = Settings.Instance.PrefixStars;
			f3.checkBox3.Checked = Settings.Instance.DontConPlayers;
			f3.checkBox4.Checked = Settings.Instance.SaveOnExit;
			f3.checkBox5.Checked = Settings.Instance.MatchFullText;
			f3.textBox3.Text = Settings.Instance.RangeCircle.ToString();
			f3.textBox4.Text = Settings.Instance.LevelOverride.ToString();
			f3.textBox5.Text = Settings.Instance.UpdateDelay.ToString();
			f3.checkBox6.Checked = Settings.Instance.DrawDirectionLines;
			f3.ShowDialog();
			Settings.Instance.IP = f3.textBox1.Text;
			Settings.Instance.Port = int.Parse(f3.textBox2.Text);
			Settings.Instance.BeepOnAlert = f3.checkBox1.Checked;
			Settings.Instance.PrefixStars = f3.checkBox2.Checked;
			Settings.Instance.RangeCircle = int.Parse(f3.textBox3.Text);
			Settings.Instance.SaveOnExit = f3.checkBox4.Checked;
			Settings.Instance.MatchFullText = f3.checkBox5.Checked;
			Settings.Instance.LevelOverride = int.Parse(f3.textBox4.Text);
			Settings.Instance.DontConPlayers = f3.checkBox3.Checked;
			Settings.Instance.UpdateDelay = int.Parse(f3.textBox5.Text);
			Settings.Instance.DrawDirectionLines = f3.checkBox6.Checked;
			mapPane1.map.fillConColors();
			savePrefs("prefs.xml");
		}

		private void menuItem6_Click(object sender, System.EventArgs e) {
			loadAlerts("filters.conf");
		}

		public string typeToString(byte Type)  {
			return Bodytypes[Type];
		}

		public void NetThread()  {

			//Byte[] read = new Byte[96];
			Byte []buffer = new Byte[100000];
			int c = 0;
			bool found = false;
			ArrayList newSpawns = new ArrayList();
			bool done = false;

			done = false;
			while (!done)  {					
				Thread.Sleep(Settings.Instance.UpdateDelay);
				s.WriteByte(0x23);
				
				int bytes = s.Read(buffer, 0, 100000);

				if (bytes > 0)  {
					int numPackets = BitConverter.ToInt32(buffer, 0);

					if (numPackets*96+4 != bytes)  {
						while (bytes < numPackets*96+4)  {
							int newbytes = s.Read(buffer, bytes, numPackets*96+4-bytes);
							bytes += newbytes;
						}
					}
					
					newSpawns.Clear();

					for (int pktc = 0; pktc < numPackets; pktc++)  {
						SPAWNINFO si = new SPAWNINFO();
						si.frombytes(buffer, 4+pktc*96);

						// Check for my special packets...
						// target
						if (si.flags == 1)  {
							mapPane1.map.curTarget = si.Name;
						}
							// Zone name
						else if (si.flags == 4)  {
							if (si.Name != curZone)  {
								string f = "./maps/";
								string fn = new string(si.Name.ToCharArray(0, strlen(si.Name)));
								f += fn;
								//f += ".map";

								// Try it as a SEQ map first
								if (!loadmap(f + ".map"))  {
								
									// If it didn't work, try a LoY map
									if (!loadmap(f + ".txt"))  {
										//... Missing map		
										//spook 2003-04-26: changed message to identify map that couldn't be found
										MessageBox.Show("Couldn't find a map for the zone: " + f, "Error");
									}
									else  {
										//mapPane1.map.fillConColors();
										curZone = si.Name;
									}
								}
								else  {
									curZone = si.Name;
								}
								// spook 2003-04-26: update map con color on zone load every time
								mapPane1.map.fillConColors();
							}					
						}
							// Player packet
						else if (si.flags == 253)  {
							mapPane1.map.playerinfo = si;
							Size sz = mapPane1.map.Size;
							mapPane1.map.offsetx = (int)((mapPane1.map.Width/2)+(si.Y*(mapPane1.map.rawscale*mapPane1.map.scale)));
							mapPane1.map.offsety = (int)((mapPane1.map.Height/2)+(si.X*(mapPane1.map.rawscale*mapPane1.map.scale)));
						}
							// Ground Item Packet
						else if (si.flags == 5)  {
							found = false;

							foreach (GroundItem gi in mapPane1.map.items)  {
								if (gi.Name == si.Name && gi.X == si.X && gi.Y == si.Y && gi.Z == si.Z && gi.ID == si.SpawnID)  {
									found = true;
									gi.gone = 0;
									break;
								}
							}

							if (!found)  {
								GroundItem gi = new GroundItem();
								gi.X = si.X;
								gi.Y = si.Y;
								gi.Z = si.Z;

								gi.Name = si.Name;
								gi.ID = si.SpawnID;

								mapPane1.map.items.Add(gi);
							}

						}
							// Normal spawn packet
						else  {	
							found = false;
							lock(mapPane1.map.mobs)  {						
								// See if we've already got this one
								for (c = 0; c < mapPane1.map.mobs.Count; c++)  {
									if (si.SpawnID == ((SPAWNINFO)mapPane1.map.mobs[c]).SpawnID)  {
										((SPAWNINFO)mapPane1.map.mobs[c]).gone = 0;

										// Check if the state has changed in any important way
										if (si.Hide != ((SPAWNINFO)mapPane1.map.mobs[c]).Hide)  {
											((SPAWNINFO)mapPane1.map.mobs[c]).Hide = si.Hide;
											foreach(ListViewItem it in listBox1.Items)  {
												if (it.Text == si.Name)  {
													if (si.Hide == 1)
														it.SubItems[6].Text = "Hidden";
													else if (si.Hide == 2)
														it.SubItems[6].Text = "Invis";
													else if (si.Hide == 3)
														it.SubItems[6].Text = "IVU";
													else if (si.Hide == 4)
														it.SubItems[6].Text = "IVA";
													else 
														it.SubItems[6].Text = "Visible";
												}
											}
										}
										if (si.Heading != ((SPAWNINFO)mapPane1.map.mobs[c]).Heading)  {
											((SPAWNINFO)mapPane1.map.mobs[c]).Heading = si.Heading;
										}
										if (si.SpeedRun != ((SPAWNINFO)mapPane1.map.mobs[c]).SpeedRun)  {
											((SPAWNINFO)mapPane1.map.mobs[c]).SpeedRun = si.SpeedRun;
											foreach(ListViewItem it in listBox1.Items)  
												if (it.Text == si.Name)  
													it.SubItems[7].Text = si.SpeedRun.ToString();
										}
										if (si.X != ((SPAWNINFO)mapPane1.map.mobs[c]).X)  {
											((SPAWNINFO)mapPane1.map.mobs[c]).X = si.X;
											//											foreach(ListViewItem it in listBox1.Items)  
											//												if (it.Text == si.Name)  
											//													it.SubItems[10].Text = si.X.ToString();
										}
										if (si.Y != ((SPAWNINFO)mapPane1.map.mobs[c]).Y)  {
											((SPAWNINFO)mapPane1.map.mobs[c]).Y = si.Y;
											//											foreach(ListViewItem it in listBox1.Items)  
											//												if (it.Text == si.Name)  
											//													it.SubItems[11].Text = si.Y.ToString();
										}
										if (si.Z != ((SPAWNINFO)mapPane1.map.mobs[c]).Z)  {
											((SPAWNINFO)mapPane1.map.mobs[c]).Z = si.Z;
											//											foreach(ListViewItem it in listBox1.Items)  
											//												if (it.Text == si.Name)  
											//													it.SubItems[12].Text = si.Z.ToString();
										}

										//	mapPane1.map.mobs[c] = si;
										found = true;
										break;	
									}
								}
							}

							// if it's not in the list already add it
							if (!found)  {
								bool alert = false;

								if (si.Name != "")  {
									string t = si.Name;
									foreach (string str in alerts)  {
										bool matched = false;

										if (Settings.Instance.MatchFullText)  {
											if (si.Name == str)
												matched = true;
										}
										else  {
											if (isSubstring(si.Name, str))  
												matched = true;
										}

										if (matched)  {
											if (Settings.Instance.PrefixStars)
												t = "*** " + si.Name;
											if (Settings.Instance.BeepOnAlert)
												Beep(300, 100);
											alert = true;
											si.isAlert = true;
											break;
										}
									}

									ListViewItem item1 = new ListViewItem(t,0);

									item1.SubItems.Add(si.Level.ToString());
									item1.SubItems.Add(classNumToString(si.Class));		
									item1.SubItems.Add(raceNumtoString(si.Race));  
									item1.SubItems.Add(si.Lastname);	
									item1.SubItems.Add(typeToString(si.Type));
									if (si.Hide == 1)
										item1.SubItems.Add("Hidden");
									else if (si.Hide == 2)
										item1.SubItems.Add("Invis");
									else if (si.Hide == 3)
										item1.SubItems.Add("IVU");
									else if (si.Hide == 4)
										item1.SubItems.Add("IVA");
									else
										item1.SubItems.Add("Visible");
									item1.SubItems.Add(si.SpeedRun.ToString());
									item1.SubItems.Add(si.SpawnID.ToString());
									DateTime dt = DateTime.Now;
									item1.SubItems.Add(dt.ToLongTimeString());											
									item1.SubItems.Add(si.X.ToString());											
									item1.SubItems.Add(si.Y.ToString());											
									item1.SubItems.Add(si.Z.ToString());		
									newSpawns.Add(item1);


									if (si.Type != 2)  {
										item1.ForeColor = mapPane1.map.conColors[si.Level].Color;
										// Change the colors to be more visible on white
										if (item1.ForeColor == Color.White)
											item1.ForeColor = Color.Black;
										if (item1.ForeColor == Color.Yellow)
											item1.ForeColor = Color.Goldenrod;
										if (item1.ForeColor == Color.Cyan)
											item1.ForeColor = Color.LightBlue;
									}
									else  {
										item1.ForeColor = Color.Gray;
									}

									if (alert)
										item1.Font = new Font(item1.Font.FontFamily, 7, FontStyle.Bold);

									si.lvi = item1;
									si.gone = 0;
									lock(mapPane1.map.mobs)  {
										mapPane1.map.mobs.Add(si);
									}
								}
							}
						}
					}

					if (newSpawns.Count > 0)  {
						ListViewItem[] items = new ListViewItem[newSpawns.Count];
						int d = 0;
						foreach(ListViewItem i in newSpawns)
							items[d++] = i;
						lock (listBox1)  {
							listBox1.Items.AddRange(items);
						}
					}

					checkMobs();
					mapPane1.map.Invalidate();
				}
			}
		}

		private void Form1_Load(object sender, System.EventArgs e) {
		
		}

		public string classNumToString(int num)  {
			return Classes[num];
		}

		public string raceNumtoString(uint num)  {
			return Races[num];
		}

		// This SUCKS, there's a better way to handle this
		private void checkMobs()  {
			bool done = false;

			lock (mapPane1.map.items)  {
				if (mapPane1.map.items.Count > 0)  {
					foreach (GroundItem gi in mapPane1.map.items)  {
						if (gi.gone != ditchGone)
							gi.gone++;
					}

					while (!done)  {
						int c = 0;
						for (c = 0; c < mapPane1.map.items.Count; c++)  {
							GroundItem gi = (GroundItem)mapPane1.map.items[c];

							if (gi.gone == ditchGone)  {
								mapPane1.map.items.RemoveAt(c);
								break;
							}
						}

						if (c == mapPane1.map.items.Count)
							done = true;
					}
				}
			}


			lock(mapPane1.map.mobs)  {
				foreach(SPAWNINFO sp in mapPane1.map.mobs)  {
					if (sp.gone != ditchGone)
						sp.gone++;
					else if (sp.Name == "" || sp.Level == 0 || sp.Race == 0 || classNumToString(sp.Class) == "Unknown" || this.raceNumtoString(sp.Race) == "Unknown")
						sp.gone = ditchGone;
				}			
		
				while (!done)  {
					int c;
					for (c = 0; c < mapPane1.map.mobs.Count; c++)  {
						SPAWNINFO sp = (SPAWNINFO)mapPane1.map.mobs[c];
						if (sp.gone == ditchGone)  {
							lock(listBox1)  {
								for (int d = 0; d < listBox1.Items.Count; d++)  {
									if (listBox1.Items[d].SubItems[0].Text == sp.Name || listBox1.Items[d].SubItems[0].Text == "*** " + sp.Name)										
										listBox1.Items.RemoveAt(d);
								}
							}
							mapPane1.map.mobs.RemoveAt(c);
							break;
						}
					}
					
					if (c == mapPane1.map.mobs.Count)
						done = true;
				}
			}
		}

		private void listBox1_SelectedIndexChanged(object sender, System.EventArgs e) {
			ListView.SelectedIndexCollection sel = listBox1.SelectedIndices;

			if (sel.Count > 0) {
				lock(listBox1)
					mapPane1.map.selectedID = int.Parse(listBox1.Items[sel[0]].SubItems[8].Text);
				mapPane1.map.Invalidate();
			}

		}

		private void menuItem8_Click(object sender, System.EventArgs e) {
			lock(mapPane1.map.mobs)  {
				lock(listBox1)  {
					mapPane1.map.mobs.Clear();
					listBox1.Items.Clear();
				}
			}
			lock(mapPane1.map.items)
				mapPane1.map.items.Clear();
		}

		private void menuItem11_Click(object sender, System.EventArgs e) {
			saveAlerts("filters.conf");
		}

		private void menuItem10_Click(object sender, System.EventArgs e) {
			Form4 f4 = new Form4();

			f4.ShowDialog();

			if (!f4.cancelled) 
				alerts.Add(f4.textBox1.Text);
		}

		private void listBox1_ColumnClick(object sender, ColumnClickEventArgs e)  {
			// Put it in descending order if the column was already the one that was being sorted			
			if (curColumn == e.Column)  {
				if (curDescend)
					curDescend = false;
				else 
					curDescend = true;
			}
			else
				curDescend = false;

			curColumn = e.Column;	
			listBox1.ListViewItemSorter = new ListBoxComparer(listBox1.Items, curDescend, e.Column);

		}

		private void fontDialog1_Apply_1(object sender, System.EventArgs e) {
			listBox1.Font = fontDialog1.Font;
			
		}

		private void menuItem7_Click(object sender, System.EventArgs e) {
			fontDialog1.ShowApply = true;
			if(fontDialog1.ShowDialog() != DialogResult.Cancel ) {
				listBox1.Font = fontDialog1.Font ;
			}		
		}

		private void menuItem12_Click(object sender, System.EventArgs e) {			
			if(colorDialog1.ShowDialog() != DialogResult.Cancel ) {
				Settings.Instance.BackColor = colorDialog1.Color;
				mapPane1.map.Invalidate();
			}		
		}

		private void menuItem13_Click(object sender, System.EventArgs e) {
			savePrefs("prefs.xml");
		}

		private void Form1_Closing(object sender, CancelEventArgs e)  {
			if (Settings.Instance.SaveOnExit)
				savePrefs("prefs.xml");
		}

		private void menuItem14_Click(object sender, System.EventArgs e) {
			DateTime dt = DateTime.Now;			

			string filename = mapPane1.map.shortname + " - ";
			filename += dt.Month.ToString() + "-" + dt.Day + "-" + dt.Year + " " + dt.Hour + "-" + dt.Minute + ".txt";

			StreamWriter sw = new StreamWriter(filename, false);

			foreach (SPAWNINFO si in mapPane1.map.mobs)  {
				string invis = "";

				if (si.Hide == 1)
					invis = "Hidden";
				else if (si.Hide == 2)
					invis = "Invis";
				else if (si.Hide == 3)
					invis = "IVU";
				else if (si.Hide == 4)
					invis = "IVA";
				else 
					invis = "Visible";

				sw.WriteLine("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\t{9}\t{10}", si.Name, si.Level, Classes[si.Class], 
					Races[si.Race], si.Lastname, Bodytypes[si.Type], invis, si.SpeedRun, si.SpawnID, si.X, si.Y, si.Z);
			}

			sw.Close();
		}

		private void timer1_Tick(object sender, System.EventArgs e) {
			//s.Read(								
		}

		private void menuItem15_Click(object sender, System.EventArgs e) {
			if (menuItem15.Checked)
				menuItem15.Checked = false;
			else
				menuItem15.Checked = true;
			Settings.Instance.DepthFilter = menuItem15.Checked;
			mapPane1.map.Invalidate();
		}

	/*	private void menuItem16_Click(object sender, System.EventArgs e) {
			if (!menuItem16.Checked)
				menuItem16.Checked = true;
			else 
				menuItem16.Checked = false;
			Settings.Instance.ShowNPCs = menuItem16.Checked;

			lock(mapPane1.map.mobs)  {
				lock(listBox1)  {
					listBox1.BeginUpdate();
				
					mapPane1.map.mobs.Clear();
					listBox1.Items.Clear();

					listBox1.EndUpdate();
				}
			}
		}

		private void menuItem17_Click(object sender, System.EventArgs e) {
			if (!menuItem17.Checked)
				menuItem17.Checked = true;
			else 
				menuItem17.Checked = false;
			Settings.Instance.ShowNPCs = menuItem17.Checked;

			lock(mapPane1.map.mobs)  {
				lock(listBox1)  {
					listBox1.BeginUpdate();
				
					mapPane1.map.mobs.Clear();
					listBox1.Items.Clear();

					listBox1.EndUpdate();
				}
			}
		}

		private void menuItem18_Click(object sender, System.EventArgs e) {
			if (!menuItem18.Checked)
				menuItem18.Checked = true;
			else 
				menuItem18.Checked = false;
			Settings.Instance.ShowNPCs = menuItem18.Checked;
		
			lock(mapPane1.map.mobs)  {
				lock(listBox1)  {
					listBox1.BeginUpdate();
				
					mapPane1.map.mobs.Clear();
					listBox1.Items.Clear();

					listBox1.EndUpdate();
				}
			}
		}*/

	}
}

